package sun.net;

import java.util.Collections;
import java.util.Iterator;
import java.util.HashSet;
import java.net.SocketException;
import java.io.FileDescriptor;
import java.net.SocketOption;
import java.util.Set;

public abstract class ExtendedSocketOptions
{
    public static final short SOCK_STREAM = 1;
    public static final short SOCK_DGRAM = 2;
    private final Set<SocketOption<?>> options;
    private final Set<SocketOption<?>> datagramOptions;
    private final Set<SocketOption<?>> clientStreamOptions;
    private final Set<SocketOption<?>> serverStreamOptions;
    private static volatile ExtendedSocketOptions instance;
    
    public final boolean isOptionSupported(SocketOption<?> socketOption) {
        return this.options().contains(socketOption);
    }
    
    public final Set<SocketOption<?>> options() {
        return this.options;
    }
    
    public static Set<SocketOption<?>> serverSocketOptions() {
        return getInstance().options0((short)1, true);
    }
    
    public static Set<SocketOption<?>> clientSocketOptions() {
        return getInstance().options0((short)1, false);
    }
    
    public static Set<SocketOption<?>> datagramSocketOptions() {
        return getInstance().options0((short)2, false);
    }
    
    private static boolean isDatagramOption(SocketOption<?> socketOption) {
        return !socketOption.name().startsWith("TCP_");
    }
    
    private static boolean isStreamOption(SocketOption<?> socketOption, boolean b) {
        return (!b || !"SO_FLOW_SLA".equals(socketOption.name())) && !socketOption.name().startsWith("UDP_");
    }
    
    private Set<SocketOption<?>> options0(short n, boolean b) {
        switch (n) {
            case 2: {
                return this.datagramOptions;
            }
            case 1: {
                if (b) {
                    return this.serverStreamOptions;
                }
                return this.clientStreamOptions;
            }
            default: {
                throw new IllegalArgumentException("Invalid socket option type");
            }
        }
    }
    
    public abstract void setOption(FileDescriptor p0, SocketOption<?> p1, Object p2) throws SocketException;
    
    public abstract Object getOption(FileDescriptor p0, SocketOption<?> p1) throws SocketException;
    
    protected ExtendedSocketOptions(Set<SocketOption<?>> options) {
        this.options = options;
        HashSet<SocketOption> c = new HashSet();
        HashSet<SocketOption> c2 = new HashSet();
        HashSet<SocketOption> c3 = new HashSet();
        for (SocketOption socketOption : options) {
            if (isDatagramOption(socketOption)) {
                c.add(socketOption);
            }
            if (isStreamOption(socketOption, true)) {
                c2.add(socketOption);
            }
            if (isStreamOption(socketOption, false)) {
                c3.add(socketOption);
            }
            else {
                continue;
            }
        }
        this.datagramOptions = new HashSet(c);
        this.serverStreamOptions = new HashSet(c2);
        this.clientStreamOptions = new HashSet(c3);
    }
    
    public static final ExtendedSocketOptions getInstance() {
        return ExtendedSocketOptions.instance;
    }
    
    public static final void register(ExtendedSocketOptions instance) {
        if (ExtendedSocketOptions.instance != null) {
            throw new InternalError("Attempting to reregister extended options");
        }
        ExtendedSocketOptions.instance = instance;
        return;
    }
    
    static {
        try {
            Class.forName("jdk.net.ExtendedSocketOptions");
        }
        catch (ClassNotFoundException ex) {
            ExtendedSocketOptions.instance = new ExtendedSocketOptions.NoExtendedSocketOptions();
        }
    }
    
    static final class NoExtendedSocketOptions extends ExtendedSocketOptions
    {
        NoExtendedSocketOptions() {
            super(Collections.emptySet());
        }
        
        @Override
        public void setOption(FileDescriptor fileDescriptor, SocketOption<?> socketOption, Object o) throws SocketException {
            throw new UnsupportedOperationException("no extended options: " + socketOption.name());
        }
        
        @Override
        public Object getOption(FileDescriptor fileDescriptor, SocketOption<?> socketOption) throws SocketException {
            throw new UnsupportedOperationException("no extended options: " + socketOption.name());
        }
    }
}
